@@ -3,10 +3,14 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+from ._version import get_versions
+__version__ = get_versions()['version']
+del get_versions
+
 from .core.alignment import align, broadcast, broadcast_arrays
 from .core.common import full_like, zeros_like, ones_like
 from .core.combine import concat, auto_combine
-from .core.computation import apply_ufunc, where
+from .core.computation import apply_ufunc, where, dot
 from .core.extensions import (register_dataarray_accessor,
                               register_dataset_accessor)
 from .core.variable import as_variable, Variable, IndexVariable, Coordinate
@@ -22,13 +26,8 @@ from .backends.zarr import open_zarr
 
 from .conventions import decode_cf, SerializationWarning
 
-try:
-    from .version import version as __version__
-except ImportError:  # pragma: no cover
-    raise ImportError('xarray not properly installed. If you are running from '
-                      'the source directory, please instead create a new '
-                      'virtual environment (using conda or virtualenv) and '
-                      'then install it in-place by running: pip install -e .')
+from .coding.cftimeindex import CFTimeIndex
+
 from .util.print_versions import show_versions
 
 from . import tutorial
